/* exported init */
/* exported enable */
/* exported disable */

'use strict';

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const { Clutter, Gio, GLib, GObject, St } = imports.gi;
const { main, panelMenu } = imports.ui;

const LatencyIndicator = GObject.registerClass(
	class LatencyIndicator extends panelMenu.Button {
		_init() {
			super._init(0, "LatencyIndicator", false);

			this.icon = new St.Icon({
				gicon: new Gio.ThemedIcon({ name: "network-transmit-receive" }),
				style_class: "system-status-icon"
			});

			this.label = new St.Label({
				y_expand: true,
				y_align: Clutter.ActorAlign.CENTER
			});
			this.label.text = "...";
			this._update();

			this.box = new St.BoxLayout();
			this.box.add(this.icon);
			this.box.add(this.label);

			this.add_child(this.box);
		}

		_update() {
			try {
				let process = Gio.Subprocess.new(
					["ping", "-c", "3", "1.1.1.1"],
					Gio.SubprocessFlags.STDOUT_PIPE | Gio.SubprocessFlags.STDERR_PIPE
				);

				process.communicate_utf8_async(null, null, (process, result) => {
					try {
						let [, stdout, stderr] = process.communicate_utf8_finish(result);
						let exit_status = process.get_exit_status();

						if (exit_status !== 0) {
							if (exit_status == 1) this.label.text = "Timeout";
							else this.label.text = "Error";
							throw new Error(stderr);
						}

						let [, rttAvg] = this.getPingRTT(stdout);

						this.label.text = `${rttAvg} ms`;
					} catch (error) {
						logError(error);
					}
				});
			} catch (error) {
				logError(error);
			} finally {
				this._timeout = GLib.timeout_add_seconds(GLib.PRIORITY_DEFAULT, 3, this._update.bind(this));
			}

			return GLib.SOURCE_REMOVE;
		}

		getPingRTT(pingResult) {
			pingResult = pingResult.trim().split(/\n/);
			pingResult = pingResult[pingResult.length - 1].toString(); // rtt min/avg/max/mdev = 13.109/21.812/38.923/9.078 ms
			pingResult = pingResult.split(/\s/)[3].toString();         // 13.109/21.812/38.923/9.078
			pingResult = pingResult.split(/\//);

			let rttMin = Math.round(pingResult[0]);
			let rttAvg = Math.round(pingResult[1]);
			let rrtMax = Math.round(pingResult[2]);

			return [rttMin, rttAvg, rrtMax];
		}

		_onDestroy() {
			if (this._timeout) GLib.source_remove(this._timeout);

			super._onDestroy();
		}
	}
)


let latencyIndicator;

function init() {
	log(`Initializing ${Me.metadata.name} version ${Me.metadata.version}`);
}

function enable() {
	latencyIndicator = new LatencyIndicator();
	main.panel.addToStatusArea('latencyIndicator', latencyIndicator);
}

function disable() {
	latencyIndicator.destroy();
}
